#-*- coding: utf-8 -*-
'''
tcptest.redis
~~~~~~~~~~~~~
'''

import tempfile
import time
import os

from . import TestServer


class Server(TestServer):
    '''Redis Test Server'''

    def __init__(self, settings=None, *args, **kw):
        super(Server, self).__init__(*args, **kw)
        self.settings = settings or {}

    def build_command(self):
        args = ['redis-server']
        for (k, v) in self.settings.items():
            args.append('--%s' % k)
            args.extend(str(v).split(' '))
        return tuple(args)

    def _before_start(self):
        self.settings['port'] = self.port

        if 'databases' not in self.settings:
            self.settings['databases'] = 16

        if 'dir' not in self.settings:
            self.settings['dir'] = tempfile.gettempdir()

        if 'dbfilename' not in self.settings:
            now = int(time.time() * 1000)
            self.settings['dbfilename'] = 'redis-%d.rdb' % now

    def _after_stop(self):
        dbfilename = os.path.join(self.settings['dir'],
                                  self.settings['dbfilename'])
        if os.path.exists(dbfilename):
            os.remove(dbfilename)



#-*- coding: utf-8 -*-
'''
tcptest
~~~~~~~

tcptest is following Perl's "Test::TCP".

It has the methods like original one, but also has another TestServer
implementation.

This package also includes tcptest.memcached module like "Test::Memcached".

- http://search.cpan.org/~tokuhirom/Test-TCP/
- http://search.cpan.org/~dmaki/Test-Memcached/

:author: nekoya <http://nekoya.github.io/>
:license: the MIT License: http://www.opensource.org/licenses/mit-license.php
'''

__version__ = '0.3.0'

import socket
import subprocess
import time


def empty_port():
    '''Find an empty port
    Returns:
        <int> empty port
    '''
    s = socket.socket()
    s.bind(('', 0))
    (host, port) = s.getsockname()
    s.close()
    return port


def wait_port(port, timeout=3.0):
    '''Wait until the port can use
    Args:
        <int> port
        <float> timeout(optional)
    '''
    begin = time.time()

    def get_rest_time():
        return max(timeout - (time.time() - begin), 0)

    wait = 0.0001
    while 1:
        try:
            _check_port(port, timeout=get_rest_time())
            break
        except Exception, e:
            if get_rest_time() == 0:
                raise Exception('connect to port:%s timed out. %s' % (port, e))
            wait *= 2
            time.sleep(wait)


def _check_port(port, timeout):
    sock = socket.socket()
    if timeout is not None:
        sock.settimeout(timeout)
    sock.connect(('127.0.0.1', port))
    sock.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
    sock.close()


class TestServer(object):
    '''Test server abstract class'''

    def __init__(self, timeout=3.0, res=None):
        '''
        Args:
            <float> timeout(optional)
            <dict> stdout/stderr buffer(optional)
        '''
        self.timeout = timeout
        self.res = res

    def build_command(self):
        '''
        Returns:
            <tuple> arguments for popen
        '''
        return ()

    def start(self):
        '''start test server'''
        self.port = empty_port()
        self._before_start()
        cmd = self.build_command()
        self._proc = subprocess.Popen(cmd,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE)
        try:
            wait_port(port=self.port, timeout=self.timeout)
        except Exception, e:
            self.stop()
            raise e
        self._after_start()

    def stop(self):
        '''stop test server'''
        self._before_stop()
        self._proc.terminate()
        self._wait()
        self._after_stop()

    def _before_start(self):
        '''hook event'''
        pass

    def _after_start(self):
        '''hook event'''
        pass

    def _before_stop(self):
        '''hook event'''
        pass

    def _after_stop(self):
        '''hook event'''
        pass

    def _wait(self):
        try:
            (out, err) = self._proc.communicate()
            if self.res is not None:
                self.res['stdout'] = out
                self.res['stderr'] = err
        except KeyboardInterrupt:
            pass

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, etype, error, exc_info):
        self.stop()



from setuptools import setup

setup(
    name='tcptest',
    packages=['tcptest'],
    version='0.3.0',
    author='Ryo Miyake',
    author_email='ryo.studiom@gmail.com',
    description="Testing TCP program, following the Perl's Test::TCP "
                "include memcached and redis test server.",
    long_description=open('README.rst').read(),
    url='http://github.com/nekoya/python-tcptest',
    license='MIT',
    tests_require=['Nose', 'python-memcached', 'redis'],
    test_suite='nose.collector',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Testing',
    ],
)



#-*- coding: utf-8 -*-
'''
tcptest.memcached
~~~~~~~~~~~~~~~~~
'''

from . import TestServer


class Server(TestServer):
    '''Memcached Test Server'''

    def build_command(self):
        return ('memcached', '-p', str(self.port))
